unit InMemoryTable;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, DB, QGrids, QDBGrids, QExtCtrls, QDBCtrls, DBClient;

type
  TForm1 = class(TForm)
    CreateDataset: TButton;
    DeleteDataset: TButton;
    SaveDataset: TButton;
    LoadDataset: TButton;
    DataSource1: TDataSource;
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    CloseDataset: TButton;
    procedure AddAField(dataSet: TClientDataSet; fldName: String;
        fldType: TFieldType; size: Integer; reqd: Boolean);
    procedure CreateDatasetClick(Sender: TObject);
    procedure DeleteDatasetClick(Sender: TObject);
    procedure SaveDatasetClick(Sender: TObject);
    procedure LoadDatasetClick(Sender: TObject);
    procedure CloseDatasetClick(Sender: TObject);

  private
    { Private declarations }
    myDataSet: TClientDataSet;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.xfm}

procedure TForm1.AddAField(dataSet: TClientDataSet; fldName: String;
    fldType: TFieldType; size: Integer; reqd: Boolean);
var field: TField;
begin
    {This method adds a new field to the dataset}
    field := nil;
    case fldType of
        ftString: field := TStringField.Create(dataSet);
        ftSmallint: field := TSmallintField.Create(dataSet);
        ftInteger: field := TIntegerField.Create(dataSet);
        ftDate: field := TDateField.Create(dataSet);
        ftDateTime: field := TDateTimeField.Create(dataSet);
        ftFloat: field := TFloatField.Create(dataSet);
        ftBlob: field := TBlobField.Create(dataSet);
        ftBoolean: field := TBooleanField.Create(dataSet);
    end;
    if field <> nil then
    begin
        field.Name := fldName;
        field.FieldName := fldName;
        field.DataSet := dataSet;
        field.Required := reqd;
        field.Size := size;
    end;
end;
procedure TForm1.CreateDatasetClick(Sender: TObject);
begin
    {Method to create the new dataset}
    if myDataSet <> nil then
        myDataSet.Destroy;
    myDataSet := TClientDataSet.Create(Self);
    AddAField(myDataSet, 'First_Name', ftString, 25, True);
    AddAField(myDataSet, 'Last_Name', ftString, 25, True);
    AddAField(myDataSet, 'Street_Address', ftString, 40, False);
    AddAField(myDataSet, 'City_Name', ftString, 40, False);
    AddAField(myDataSet, 'State_Code', ftString, 2, False);
    AddAField(myDataSet, 'Country', ftString, 20, False);
    AddAField(myDataSet, 'Home_Phone', ftString, 15, False);
    AddAField(myDataSet, 'Work_Phone', ftString, 15, False);
    myDataSet.CreateDataSet;
    myDataSet.Open;
    DataSource1.DataSet := myDataSet;
end;

procedure TForm1.DeleteDatasetClick(Sender: TObject);
begin
    if myDataSet <> nil then
    begin
        if myDataSet.Active = True then
            myDataSet.Close;
        myDataSet.Destroy;
        myDataSet := nil;
    end;
end;

procedure TForm1.SaveDatasetClick(Sender: TObject);
var currDir: String;
begin
    { Determine the current working directory from where the program is executed}
    currDir := ExtractFileDir(Application.ExeName);

    { If the dataset is nil or is not open, then exit the procedure}
    if myDataSet = nil then
        System.Exit;
    if myDataSet.Active = False then
        System.Exit;
    { Set the initial directory of the open dialog to current directory}
    SaveDialog1.InitialDir := currDir;

    { Save the dataset with the selected name}
    if SaveDialog1.Execute = True then
        myDataSet.SaveToFile(SaveDialog1.FileName);

end;

procedure TForm1.LoadDatasetClick(Sender: TObject);
var currDir, fileName: String;
begin
    { if the dataset is not created yet, create one}
    if myDataSet = nil then
        myDataSet := TClientDataSet.Create(Self);

    { if the dataset is already open, close it first}
    if myDataSet.Active = True then
        myDataSet.Close;

    { Determine the current working directory from where the program is executed}
    currDir := ExtractFileDir(Application.ExeName);

    { Set the initial directory of the open dialog to current directory}
    OpenDialog1.InitialDir := currDir;

    { Load the dataset from the selected file}
    if OpenDialog1.Execute = True then
    begin
        fileName := OpenDialog1.FileName;
        myDataSet.LoadFromFile(fileName);
        DataSource1.DataSet := myDataSet;
    end;
end;

procedure TForm1.CloseDatasetClick(Sender: TObject);
begin
    if ((myDataSet <> nil) and (myDataSet.Active = True)) then
        myDataSet.Close;
end;

end.
